import javax.swing.*;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.ActionEvent;

public class MandelbrotFrame extends JFrame implements KeyListener {
    Mandelbrot mandel;
    SwingWorker worker;
    BufferedImage image;
    JPanel panel;

    public MandelbrotFrame(int largeur, int hauteur) {
        super("Ensemble de Mandelbrot");
        this.setSize(largeur, hauteur+20);
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    
        this.mandel = new Mandelbrot(largeur, hauteur);
        this.image = new BufferedImage(largeur, hauteur, BufferedImage.TYPE_INT_RGB);
        this.panel = new JPanel() {
            public void paintComponent(Graphics g) {
                super.paintComponent(g);
                g.drawImage(image, 0, 0, this);
            }
        };
        panel.setFocusTraversalKeysEnabled(false);
        panel.addKeyListener(this);
        panel.setFocusable(true);
        panel.requestFocus();
        this.getContentPane().add(panel);
        this.worker = (SwingWorker) new CalculeImageWorker();

        this.metAJourTitre();
        this.setVisible(true);
        this.worker.execute();
    }
    
    // Implementation de KeyListener
    public void keyPressed(KeyEvent e) {
        // p : augmente le nombre maximal d'iterations
        if(e.getKeyChar() == 'p') {
            mandel.nbIterMax += 50;
		    if(worker != null) worker.cancel(true);
            worker = (SwingWorker) new CalculeImageWorker();
            worker.execute();
        }
        // m : diminue le nombre maximal d'iterations
        if(e.getKeyChar()=='m') {
            mandel.nbIterMax -= 50;
		    if(worker != null) worker.cancel(true);
            worker = (SwingWorker) new CalculeImageWorker();
            worker.execute();
        }
	    // +: zoome
        if(e.getKeyChar() == '+') {
            mandel.zoom *= .9;
            if(worker != null) worker.cancel(true);
                worker = (SwingWorker) new CalculeImageWorker();
            worker.execute();
        }
        // -: dezoome
        if(e.getKeyChar() == '-') {
            mandel.zoom /= .9;
		    if(worker != null) worker.cancel(true);
            worker = (SwingWorker) new CalculeImageWorker();
            worker.execute();
        }
        // up: increase cy
        if(e.getKeyCode() == KeyEvent.VK_UP) {
            mandel.ordOrigine += .02*mandel.zoom;
		    if(worker != null) worker.cancel(true);
            worker = (SwingWorker) new CalculeImageWorker();
            worker.execute();
        }
        // down: decrease cy
        if(e.getKeyCode() == KeyEvent.VK_DOWN) {
            mandel.ordOrigine -= .02*mandel.zoom;
            if(worker != null) worker.cancel(true);
                worker = (SwingWorker) new CalculeImageWorker();
            worker.execute();
        }
        // left: decrease cx
        if(e.getKeyCode()==KeyEvent.VK_LEFT) {
            mandel.absOrigine -= .02*mandel.zoom;
            if(worker != null) worker.cancel(true);
                worker = (SwingWorker) new CalculeImageWorker();
            worker.execute();
        }
        // right: increase cx
        if(e.getKeyCode() == KeyEvent.VK_RIGHT) {
            mandel.absOrigine += .02*mandel.zoom;
            if(worker != null) worker.cancel(true);
                worker = (SwingWorker) new CalculeImageWorker();
            worker.execute();
        }
        metAJourTitre();
    }
    void metAJourTitre() {
        this.setTitle("Ensemble de Mandelbrot ("+mandel.absOrigine+", "+mandel.ordOrigine+")"); 
    }
    public void keyTyped(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}
  
    public static void main(String[] args) {
        new MandelbrotFrame(600, 600);
        
        System.out.println("Fleches pour se deplacer.");
        System.out.println("+/- pour zoomer ou dezoomer");
        System.out.println("p/m pour aumenter/diminuer le nombre d'iterations maximal.");
    }
  
    class CalculeImageWorker extends SwingWorker<Void, Void> {
        public Void doInBackground() {
            mandel.remplitImage(image);
            repaint();
            return null;
        }
    }
}
